@@ -23,9 +23,9 @@
  *                                                                          *
  ****************************************************************************/
 
-/* This is the C header corresponding to the Ada package specification for
-   Atree. It also contains the implementations of inlined functions from the
-   package body for Atree.  It was generated manually from atree.ads and
+/* This is the C header that corresponds to the Ada package specification for
+   Atree.  It also contains the implementation of inlined functions from the
+   package body for Atree.  It was created manually from atree.ads and
    atree.adb and must be kept synchronized with changes in these files.
 
    Note that only routines for reading the tree are included, since the tree
