@@ -34,7 +34,9 @@
 #define ATTRIBUTE_UNUSED __attribute__((unused))
 
 /* Ensure access to errno is thread safe.  */
+#ifndef _REENTRANT
 #define _REENTRANT
+#endif
 #define _THREAD_SAFE
 
 #include "gsocket.h"
@@ -704,12 +706,6 @@ __gnat_servent_s_proto (struct servent * s)
 
 #if defined(AF_INET6) && !defined(__rtems__)
 
-#if defined (__vxworks)
-#define getaddrinfo ipcom_getaddrinfo
-#define getnameinfo ipcom_getnameinfo
-#define freeaddrinfo ipcom_freeaddrinfo
-#endif
-
 int __gnat_getaddrinfo(
   const char *node,
   const char *service,
@@ -803,4 +799,15 @@ const char * __gnat_gai_strerror(int errcode) {
 
 #endif
 
+int __gnat_minus_500ms() {
+#if defined (_WIN32)
+  // Windows Server 2019 and Windows 8.0 do not need 500 millisecond socket
+  // timeout correction.
+  return !(IsWindows8OrGreater() && !IsWindowsServer()
+           || IsWindowsVersionOrGreater(10, 0, 17763));
+#else
+   return 0;
+#endif
+}
+
 #endif /* defined(HAVE_SOCKETS) */
