@@ -1,5 +1,5 @@
 /* Branch Target Identification for AArch64 architecture.
-   Copyright (C) 2019 Free Software Foundation, Inc.
+   Copyright (C) 2019-2020 Free Software Foundation, Inc.
    Contributed by Arm Ltd.
 
    This file is part of GCC.
@@ -95,7 +95,7 @@ static bool
 aarch64_pac_insn_p (rtx x)
 {
   if (!INSN_P (x))
-    return x;
+    return false;
 
   subrtx_var_iterator::array_type array;
   FOR_EACH_SUBRTX_VAR (iter, array, PATTERN (x), ALL)
@@ -106,7 +106,9 @@ aarch64_pac_insn_p (rtx x)
 	  int unspec_val = XINT (sub, 1);
 	  switch (unspec_val)
 	    {
-	    case UNSPEC_PACISP:
+	    case UNSPEC_PACIASP:
+            /* fall-through.  */
+            case UNSPEC_PACIBSP:
 	      return true;
 
 	    default:
