@@ -238,6 +238,9 @@ build_frontend_type (tree type)
       sdecl->type->ctype = type;
       sdecl->type->merge2 ();
 
+      /* Add both named and anonymous fields as members of the struct.
+	 Anonymous fields still need a name in D, so call them "__pad%d".  */
+      int anonfield_id = 0;
       sdecl->members = new Dsymbols;
 
       for (tree field = TYPE_FIELDS (type); field; field = DECL_CHAIN (field))
@@ -249,12 +252,19 @@ build_frontend_type (tree type)
 	      return NULL;
 	    }
 
-	  Identifier *fident
-	    = Identifier::idPool (IDENTIFIER_POINTER (DECL_NAME (field)));
+	  Identifier *fident;
+	  if (DECL_NAME (field) == NULL_TREE)
+	    fident = Identifier::generateId ("__pad", anonfield_id++);
+	  else
+	    {
+	      const char *name = IDENTIFIER_POINTER (DECL_NAME (field));
+	      fident = Identifier::idPool (name);
+	    }
+
 	  VarDeclaration *vd = VarDeclaration::create (Loc (), ftype, fident,
 						       NULL);
 	  vd->parent = sdecl;
-	  vd->offset = tree_to_uhwi (DECL_FIELD_OFFSET (field));
+	  vd->offset = tree_to_uhwi (byte_position (field));
 	  vd->semanticRun = PASSsemanticdone;
 	  vd->csym = field;
 	  sdecl->members->push (vd);
@@ -1201,5 +1211,20 @@ d_builtin_function (tree decl)
   return decl;
 }
 
+/* Same as d_builtin_function, but used to delay putting in back-end builtin
+   functions until the ISA that defines the builtin has been declared.
+   However in D, there is no global namespace.  All builtins get pushed into the
+   `gcc.builtins' module, which is constructed during the semantic analysis
+   pass, which has already finished by the time target attributes are evaluated.
+   So builtins are not pushed because they would be ultimately ignored.
+   The purpose of having this function then is to improve compile-time
+   reflection support to allow user-code to determine whether a given back end
+   function is enabled by the ISA.  */
+
+tree
+d_builtin_function_ext_scope (tree decl)
+{
+  return decl;
+}
 
 #include "gt-d-d-builtins.h"
