@@ -1,5 +1,5 @@
 /* Loop interchange.
-   Copyright (C) 2017-2019 Free Software Foundation, Inc.
+   Copyright (C) 2017-2020 Free Software Foundation, Inc.
    Contributed by ARM Ltd.
 
 This file is part of GCC.
@@ -33,7 +33,6 @@ along with GCC; see the file COPYING3.  If not see
 #include "gimple-iterator.h"
 #include "gimplify-me.h"
 #include "cfgloop.h"
-#include "params.h"
 #include "tree-ssa.h"
 #include "tree-scalar-evolution.h"
 #include "tree-ssa-loop-manip.h"
@@ -78,14 +77,14 @@ along with GCC; see the file COPYING3.  If not see
    simple reduction of inner loop and double reduction of the loop nest.  */
 
 /* Maximum number of stmts in each loop that should be interchanged.  */
-#define MAX_NUM_STMT    (PARAM_VALUE (PARAM_LOOP_INTERCHANGE_MAX_NUM_STMTS))
+#define MAX_NUM_STMT    (param_loop_interchange_max_num_stmts)
 /* Maximum number of data references in loop nest.  */
-#define MAX_DATAREFS    (PARAM_VALUE (PARAM_LOOP_MAX_DATAREFS_FOR_DATADEPS))
+#define MAX_DATAREFS    (param_loop_max_datarefs_for_datadeps)
 
 /* Comparison ratio of access stride between inner/outer loops to be
    interchanged.  This is the minimum stride ratio for loop interchange
    to be profitable.  */
-#define OUTER_STRIDE_RATIO  (PARAM_VALUE (PARAM_LOOP_INTERCHANGE_STRIDE_RATIO))
+#define OUTER_STRIDE_RATIO  (param_loop_interchange_stride_ratio)
 /* The same as above, but we require higher ratio for interchanging the
    innermost two loops.  */
 #define INNER_STRIDE_RATIO  ((OUTER_STRIDE_RATIO) + 1)
@@ -159,7 +158,7 @@ dump_reduction (reduction_p re)
 
 /* Dump LOOP's induction IV.  */
 static void
-dump_induction (struct loop *loop, induction_p iv)
+dump_induction (class loop *loop, induction_p iv)
 {
   fprintf (dump_file, "  Induction:  ");
   print_generic_expr (dump_file, iv->var, TDF_SLIM);
@@ -172,9 +171,10 @@ dump_induction (struct loop *loop, induction_p iv)
 
 /* Loop candidate for interchange.  */
 
-struct loop_cand
+class loop_cand
 {
-  loop_cand (struct loop *, struct loop *);
+public:
+  loop_cand (class loop *, class loop *);
   ~loop_cand ();
 
   reduction_p find_reduction_by_stmt (gimple *);
@@ -188,10 +188,10 @@ struct loop_cand
   void undo_simple_reduction (reduction_p, bitmap);
 
   /* The loop itself.  */
-  struct loop *m_loop;
+  class loop *m_loop;
   /* The outer loop for interchange.  It equals to loop if this loop cand
      itself represents the outer loop.  */
-  struct loop *m_outer;
+  class loop *m_outer;
   /* Vector of induction variables in loop.  */
   vec<induction_p> m_inductions;
   /* Vector of reduction variables in loop.  */
@@ -210,7 +210,7 @@ struct loop_cand
 
 /* Constructor.  */
 
-loop_cand::loop_cand (struct loop *loop, struct loop *outer)
+loop_cand::loop_cand (class loop *loop, class loop *outer)
   : m_loop (loop), m_outer (outer), m_exit (single_exit (loop)),
     m_bbs (get_loop_body (loop)), m_num_stmts (0), m_const_init_reduc (0)
 {
@@ -240,7 +240,7 @@ loop_cand::~loop_cand ()
 /* Return single use stmt of VAR in LOOP, otherwise return NULL.  */
 
 static gimple *
-single_use_in_loop (tree var, struct loop *loop)
+single_use_in_loop (tree var, class loop *loop)
 {
   gimple *stmt, *res = NULL;
   use_operand_p use_p;
@@ -879,6 +879,7 @@ loop_cand::undo_simple_reduction (reduction_p re, bitmap dce_seeds)
   if (re->producer != NULL)
     {
       gimple_set_vuse (re->producer, NULL_TREE);
+      update_stmt (re->producer);
       from = gsi_for_stmt (re->producer);
       gsi_remove (&from, false);
       gimple_seq_add_stmt_without_update (&stmts, re->producer);
@@ -920,6 +921,7 @@ loop_cand::undo_simple_reduction (reduction_p re, bitmap dce_seeds)
   gimple_set_vdef (re->consumer, NULL_TREE);
   gimple_set_vuse (re->consumer, NULL_TREE);
   gimple_assign_set_rhs1 (re->consumer, re->next);
+  update_stmt (re->consumer);
   from = gsi_for_stmt (re->consumer);
   to = gsi_for_stmt (SSA_NAME_DEF_STMT (re->next));
   gsi_move_after (&from, &to);
@@ -950,7 +952,7 @@ free_data_refs_with_aux (vec<data_reference_p> datarefs)
 class tree_loop_interchange
 {
 public:
-  tree_loop_interchange (vec<struct loop *> loop_nest)
+  tree_loop_interchange (vec<class loop *> loop_nest)
     : m_loop_nest (loop_nest), m_niters_iv_var (NULL_TREE),
       m_dce_seeds (BITMAP_ALLOC (NULL)) { }
   ~tree_loop_interchange () { BITMAP_FREE (m_dce_seeds); }
@@ -961,10 +963,10 @@ private:
   bool valid_data_dependences (unsigned, unsigned, vec<ddr_p>);
   void interchange_loops (loop_cand &, loop_cand &);
   void map_inductions_to_loop (loop_cand &, loop_cand &);
-  void move_code_to_inner_loop (struct loop *, struct loop *, basic_block *);
+  void move_code_to_inner_loop (class loop *, class loop *, basic_block *);
 
   /* The whole loop nest in which interchange is ongoing.  */
-  vec<struct loop *> m_loop_nest;
+  vec<class loop *> m_loop_nest;
   /* We create new IV which is only used in loop's exit condition check.
      In case of 3-level loop nest interchange, when we interchange the
      innermost two loops, new IV created in the middle level loop does
@@ -1081,7 +1083,7 @@ tree_loop_interchange::interchange_loops (loop_cand &iloop, loop_cand &oloop)
     }
 
   /* Prepare niters for both loops.  */
-  struct loop *loop_nest = m_loop_nest[0];
+  class loop *loop_nest = m_loop_nest[0];
   edge instantiate_below = loop_preheader_edge (loop_nest);
   gsi = gsi_last_bb (loop_preheader_edge (loop_nest)->src);
   i_niters = number_of_latch_executions (iloop.m_loop);
@@ -1216,8 +1218,8 @@ tree_loop_interchange::map_inductions_to_loop (loop_cand &src, loop_cand &tgt)
 /* Move stmts of outer loop to inner loop.  */
 
 void
-tree_loop_interchange::move_code_to_inner_loop (struct loop *outer,
-						struct loop *inner,
+tree_loop_interchange::move_code_to_inner_loop (class loop *outer,
+						class loop *inner,
 						basic_block *outer_bbs)
 {
   basic_block oloop_exit_bb = single_exit (outer)->src;
@@ -1251,14 +1253,17 @@ tree_loop_interchange::move_code_to_inner_loop (struct loop *outer,
 	      continue;
 	    }
 
-	  if (gimple_vuse (stmt))
-	    gimple_set_vuse (stmt, NULL_TREE);
 	  if (gimple_vdef (stmt))
 	    {
 	      unlink_stmt_vdef (stmt);
 	      release_ssa_name (gimple_vdef (stmt));
 	      gimple_set_vdef (stmt, NULL_TREE);
 	    }
+	  if (gimple_vuse (stmt))
+	    {
+	      gimple_set_vuse (stmt, NULL_TREE);
+	      update_stmt (stmt);
+	    }
 
 	  reset_debug_uses (stmt);
 	  gsi_move_before (&gsi, &to);
@@ -1278,7 +1283,7 @@ tree_loop_interchange::move_code_to_inner_loop (struct loop *outer,
 	   arr[i][j - 1][k] = 0;  */
 
 static void
-compute_access_stride (struct loop *loop_nest, struct loop *loop,
+compute_access_stride (class loop *loop_nest, class loop *loop,
 		       data_reference_p dr)
 {
   vec<tree> *strides = new vec<tree> ();
@@ -1322,10 +1327,10 @@ compute_access_stride (struct loop *loop_nest, struct loop *loop,
   if (! chrec_contains_undetermined (scev))
     {
       tree sl = scev;
-      struct loop *expected = loop;
+      class loop *expected = loop;
       while (TREE_CODE (sl) == POLYNOMIAL_CHREC)
 	{
-	  struct loop *sl_loop = get_chrec_loop (sl);
+	  class loop *sl_loop = get_chrec_loop (sl);
 	  while (sl_loop != expected)
 	    {
 	      strides->safe_push (size_int (0));
@@ -1353,8 +1358,8 @@ compute_access_stride (struct loop *loop_nest, struct loop *loop,
    all data references.  If access strides cannot be computed at least
    for two levels of loop for any data reference, it returns NULL.  */
 
-static struct loop *
-compute_access_strides (struct loop *loop_nest, struct loop *loop,
+static class loop *
+compute_access_strides (class loop *loop_nest, class loop *loop,
 			vec<data_reference_p> datarefs)
 {
   unsigned i, j, num_loops = (unsigned) -1;
@@ -1392,8 +1397,8 @@ compute_access_strides (struct loop *loop_nest, struct loop *loop,
    of loops that isn't in current LOOP_NEST.  */
 
 static void
-prune_access_strides_not_in_loop (struct loop *loop_nest,
-				  struct loop *innermost,
+prune_access_strides_not_in_loop (class loop *loop_nest,
+				  class loop *innermost,
 				  vec<data_reference_p> datarefs)
 {
   data_reference_p dr;
@@ -1714,7 +1719,7 @@ public:
 	nest with LOOP.  */
 
 static bool
-proper_loop_form_for_interchange (struct loop *loop, struct loop **min_outer)
+proper_loop_form_for_interchange (class loop *loop, class loop **min_outer)
 {
   edge e0, e1, exit;
 
@@ -1813,14 +1818,14 @@ proper_loop_form_for_interchange (struct loop *loop, struct loop **min_outer)
    should be interchanged by looking into all DATAREFS.  */
 
 static bool
-should_interchange_loop_nest (struct loop *loop_nest, struct loop *innermost,
+should_interchange_loop_nest (class loop *loop_nest, class loop *innermost,
 			      vec<data_reference_p> datarefs)
 {
   unsigned idx = loop_depth (innermost) - loop_depth (loop_nest);
   gcc_assert (idx > 0);
 
   /* Check if any two adjacent loops should be interchanged.  */
-  for (struct loop *loop = innermost;
+  for (class loop *loop = innermost;
        loop != loop_nest; loop = loop_outer (loop), idx--)
     if (should_interchange_loops (idx, idx - 1, datarefs, 0, 0,
 				  loop == innermost, false))
@@ -1840,7 +1845,7 @@ tree_loop_interchange_compute_ddrs (vec<loop_p> loop_nest,
 				    vec<ddr_p> *ddrs)
 {
   struct data_reference *a, *b;
-  struct loop *innermost = loop_nest.last ();
+  class loop *innermost = loop_nest.last ();
 
   for (unsigned i = 0; datarefs.iterate (i, &a); ++i)
     {
@@ -1882,7 +1887,7 @@ tree_loop_interchange_compute_ddrs (vec<loop_p> loop_nest,
 /* Prune DATAREFS by removing any data reference not inside of LOOP.  */
 
 static inline void
-prune_datarefs_not_in_loop (struct loop *loop, vec<data_reference_p> datarefs)
+prune_datarefs_not_in_loop (class loop *loop, vec<data_reference_p> datarefs)
 {
   unsigned i, j;
   struct data_reference *dr;
@@ -1909,10 +1914,10 @@ prune_datarefs_not_in_loop (struct loop *loop, vec<data_reference_p> datarefs)
    inner loop of that basic block's father loop.  On success, return the
    outer loop of the result loop nest.  */
 
-static struct loop *
-prepare_data_references (struct loop *loop, vec<data_reference_p> *datarefs)
+static class loop *
+prepare_data_references (class loop *loop, vec<data_reference_p> *datarefs)
 {
-  struct loop *loop_nest = loop;
+  class loop *loop_nest = loop;
   vec<data_reference_p> *bb_refs;
   basic_block bb, *bbs = get_loop_body_in_dom_order (loop);
 
@@ -1976,11 +1981,11 @@ prepare_data_references (struct loop *loop, vec<data_reference_p> *datarefs)
    in interchange.  */
 
 static bool
-prepare_perfect_loop_nest (struct loop *loop, vec<loop_p> *loop_nest,
+prepare_perfect_loop_nest (class loop *loop, vec<loop_p> *loop_nest,
 			   vec<data_reference_p> *datarefs, vec<ddr_p> *ddrs)
 {
-  struct loop *start_loop = NULL, *innermost = loop;
-  struct loop *outermost = loops_for_fn (cfun)->tree_root;
+  class loop *start_loop = NULL, *innermost = loop;
+  class loop *outermost = loops_for_fn (cfun)->tree_root;
 
   /* Find loop nest from the innermost loop.  The outermost is the innermost
      outer*/
@@ -2066,7 +2071,7 @@ pass_linterchange::execute (function *fun)
     return 0;
 
   bool changed_p = false;
-  struct loop *loop;
+  class loop *loop;
   FOR_EACH_LOOP (loop, LI_ONLY_INNERMOST)
     {
       vec<loop_p> loop_nest = vNULL;
