@@ -1,5 +1,5 @@
 /* Implementation of the C API; all wrappers into the internal C++ API
-   Copyright (C) 2013-2019 Free Software Foundation, Inc.
+   Copyright (C) 2013-2020 Free Software Foundation, Inc.
    Contributed by David Malcolm <dmalcolm@redhat.com>.
 
 This file is part of GCC.
@@ -23,6 +23,8 @@ along with GCC; see the file COPYING3.  If not see
 #include "coretypes.h"
 #include "timevar.h"
 #include "typed-splay-tree.h"
+#include "cppbuiltin.h"
+#include <pthread.h>
 
 #include "libgccjit.h"
 #include "jit-recording.h"
@@ -62,6 +64,10 @@ struct gcc_jit_field : public gcc::jit::recording::field
 {
 };
 
+struct gcc_jit_bitfield : public gcc::jit::recording::bitfield
+{
+};
+
 struct gcc_jit_function : public gcc::jit::recording::function
 {
 };
@@ -554,6 +560,42 @@ gcc_jit_context_new_field (gcc_jit_context *ctxt,
   return (gcc_jit_field *)ctxt->new_field (loc, type, name);
 }
 
+/* Public entrypoint.  See description in libgccjit.h.
+
+   After error-checking, the real work is done by the
+   gcc::jit::recording::context::new_bitfield method, in
+   jit-recording.c.  */
+
+gcc_jit_field *
+gcc_jit_context_new_bitfield (gcc_jit_context *ctxt,
+			      gcc_jit_location *loc,
+			      gcc_jit_type *type,
+			      int width,
+			      const char *name)
+{
+  RETURN_NULL_IF_FAIL (ctxt, NULL, NULL, "NULL context");
+  JIT_LOG_FUNC (ctxt->get_logger ());
+  /* LOC can be NULL.  */
+  RETURN_NULL_IF_FAIL (name, ctxt, loc, "NULL name");
+  RETURN_NULL_IF_FAIL (type, ctxt, loc, "NULL type");
+  RETURN_NULL_IF_FAIL_PRINTF2 (type->is_int () || type->is_bool (),
+			       ctxt, loc,
+			       "bit-field %s has non integral type %s",
+			       name, type->get_debug_string ());
+  RETURN_NULL_IF_FAIL_PRINTF2 (
+    width > 0, ctxt, loc,
+    "invalid width %d for bitfield \"%s\" (must be > 0)",
+    width, name);
+  RETURN_NULL_IF_FAIL_PRINTF2 (
+    type->has_known_size (),
+    ctxt, loc,
+    "unknown size for field \"%s\" (type: %s)",
+    name,
+    type->get_debug_string ());
+
+  return (gcc_jit_field *)ctxt->new_bitfield (loc, type, width, name);
+}
+
 /* Public entrypoint.  See description in libgccjit.h.
 
    After error-checking, this calls the trivial
@@ -1296,6 +1338,13 @@ gcc_jit_context_new_unary_op (gcc_jit_context *ctxt,
     "unrecognized value for enum gcc_jit_unary_op: %i",
     op);
   RETURN_NULL_IF_FAIL (result_type, ctxt, loc, "NULL result_type");
+  RETURN_NULL_IF_FAIL_PRINTF3 (
+    result_type->is_numeric (), ctxt, loc,
+    "gcc_jit_unary_op %s with operand %s "
+    "has non-numeric result_type: %s",
+    gcc::jit::unary_op_reproducer_strings[op],
+    rvalue->get_debug_string (),
+    result_type->get_debug_string ());
   RETURN_NULL_IF_FAIL (rvalue, ctxt, loc, "NULL rvalue");
 
   return (gcc_jit_rvalue *)ctxt->new_unary_op (loc, op, result_type, rvalue);
@@ -1345,6 +1394,13 @@ gcc_jit_context_new_binary_op (gcc_jit_context *ctxt,
     a->get_type ()->get_debug_string (),
     b->get_debug_string (),
     b->get_type ()->get_debug_string ());
+  RETURN_NULL_IF_FAIL_PRINTF4 (
+    result_type->is_numeric (), ctxt, loc,
+    "gcc_jit_binary_op %s with operands a: %s b: %s "
+    "has non-numeric result_type: %s",
+    gcc::jit::binary_op_reproducer_strings[op],
+    a->get_debug_string (), b->get_debug_string (),
+    result_type->get_debug_string ());
 
   return (gcc_jit_rvalue *)ctxt->new_binary_op (loc, op, result_type, a, b);
 }
@@ -3121,3 +3177,47 @@ gcc_jit_context_new_rvalue_from_vector (gcc_jit_context *ctxt,
      as_vec_type,
      (gcc::jit::recording::rvalue **)elements);
 }
+
+/* A mutex around the cached state in parse_basever.
+   Ideally this would be within parse_basever, but the mutex is only needed
+   by libgccjit.  */
+
+static pthread_mutex_t version_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+struct version_info
+{
+  /* Default constructor.  Populate via parse_basever,
+     guarded by version_mutex.  */
+  version_info ()
+  {
+    pthread_mutex_lock (&version_mutex);
+    parse_basever (&major, &minor, &patchlevel);
+    pthread_mutex_unlock (&version_mutex);
+  }
+
+  int major;
+  int minor;
+  int patchlevel;
+};
+
+
+extern int
+gcc_jit_version_major (void)
+{
+  version_info vi;
+  return vi.major;
+}
+
+extern int
+gcc_jit_version_minor (void)
+{
+  version_info vi;
+  return vi.minor;
+}
+
+extern int
+gcc_jit_version_patchlevel (void)
+{
+  version_info vi;
+  return vi.patchlevel;
+}
