@@ -2,6 +2,9 @@
    result on success can be computed at compile time (they can fail).
    The calls can still be transformed into those of other functions.
    { dg-require-effective-target unwrapped }
+   { dg-require-effective-target fileio }
+   { dg-prune-output "warning: warning: \[^\n\r\]* possibly used unsafely" }
+   { dg-skip-if "requires io" { avr-*-* } }
    { dg-skip-if "requires io" { freestanding } } */
 
 #include <stdio.h>
