@@ -11,11 +11,13 @@ typedef struct locus locus;
 extern int gfc_warn (const char *, ...) __attribute__ ((__format__ (__gcc_gfc__, 1, 2))) __attribute__ ((__nonnull__));
 
 void
-foo (int i, char *s, long int l, llong ll, locus *loc)
+foo (unsigned int u, int i, char *s, unsigned long int ul, long int l,
+     llong ll, locus *loc)
 {
   /* Acceptable C90 specifiers, flags and modifiers.  */
   gfc_warn ("%%");
-  gfc_warn ("%d%i%c%s%%", i, i, i, s);
+  gfc_warn ("%u%d%i%c%s%%", u, i, i, i, s);
+  gfc_warn ("%lu%ld%li%%", ul, l, l);
 
   /* Extensions provided in gfc_warn.  */
   gfc_warn ("%C");
