@@ -1,5 +1,5 @@
 /* { dg-do compile } */
-/* { dg-options "-fno-strict-overflow -O2 -fdump-tree-final_cleanup" } */
+/* { dg-options "-fno-strict-overflow -O2 -fdump-tree-optimized" } */
 
 /* Source: Ian Lance Taylor.  */
 
@@ -14,8 +14,7 @@ foo ()
   int i, bits;
   for (i = 1, bits = 1; i > 0; i += i)
     ++bits;
-  return bits;
+  return bits - sizeof(int) * __CHAR_BIT__;
 }
 
-/* { dg-final { scan-tree-dump "return bits" "final_cleanup" } } */
-/* { dg-final { cleanup-tree-dump "final_cleanup" } } */
+/* { dg-final { scan-tree-dump "return 0" "optimized" } } */
