@@ -8,10 +8,10 @@ signed long long negdi_zero_extendsidi (unsigned int x)
 }
 /*
 Expected output:
-        rsbs    r0, r0, #0
+        rsbs    r0, r0, #0 (arm) | negs	r0, r0 (thumb2)
         sbc     r1, r1, r1
 */
-/* { dg-final { scan-assembler-times "rsb" 1 } } */
+/* { dg-final { scan-assembler-times "rsbs|negs" 1 } } */
 /* { dg-final { scan-assembler-times "sbc" 1 } } */
 /* { dg-final { scan-assembler-times "mov" 0 } } */
 /* { dg-final { scan-assembler-times "rsc" 0 } } */
