@@ -4,6 +4,11 @@ typedef short  __v2hi __attribute ((vector_size(4)));
 typedef __v2hi fract2x16;
 typedef short fract16;
 
+fract2x16 foo (fract2x16 f, short n)
+{
+  return __builtin_bfin_shl_fr2x16 (f, n);
+}
+
 int main ()
 {
   fract2x16 a, t;
@@ -11,7 +16,7 @@ int main ()
 
   a = __builtin_bfin_compose_2x16 (0xcfff, 0xffff);
 
-  t = __builtin_bfin_shl_fr2x16 (a, -4);
+  t = foo (a, -4);
   t1 = __builtin_bfin_extract_hi (t);
   t2 = __builtin_bfin_extract_lo (t);
   if (t1 != 0xfffffcff || t2 != 0xffffffff)
