@@ -1,6 +1,8 @@
 /* { dg-do compile } */
+/* { dg-options "-std=gnu99" } */
 
 #include <stdint.h>
+#include <bpf-helpers.h>
 
 void
 foo ()
@@ -8,8 +10,8 @@ foo ()
   uint32_t ret;
   void *skb;
   uint32_t hash;
-  
-  ret = __builtin_bpf_helper_set_hash (skb, hash);
+
+  ret = bpf_set_hash (skb, hash);
 }
 
 /* { dg-final { scan-assembler "call\t48" } } */
