@@ -11,6 +11,6 @@ f2 (int x, int y, int z)
   asm volatile ("" ::: "$1", "$2", "$3", "$4", "$5", "$6", "$7", "$8", "$9",
 		"$10", "$11", "$12", "$13", "$14", "$15", "$16", "$17",
 		"$18", "$19", "$20", "$21", "$22", "$23", "$24", "$25",
-		"$31");
+		"$31", "lo");
   return x - y * z;
 }
