@@ -6,7 +6,7 @@
 
 void abort ();
 
-#define N 256
+#define N 4096
 signed char ca[N] __attribute__((aligned(16)));
 signed char cb[N] __attribute__((aligned(16)));
 signed char cc[N] __attribute__((aligned(16)));
@@ -21,20 +21,22 @@ __attribute__((noinline)) void foo ()
 
 __attribute__((noinline)) void init ()
 {
-  int i;
-  for (i = 0; i < N; ++i) {
-    cb[i] = i - 128;
-    cc[i] = i/2 - 64;
+  int i, ii;
+  for (i = 0, ii = 0; i < N; ++i, ii = (ii + 1) % 128) {
+    cb[i] = ii - 128;
+    cc[i] = ii/2 - 64;
   }
 }
 
 int main ()
 {
-  int i;
+  int i, ii;
   init ();
   foo ();
-  for (i = 0; i < N; ++i)
-    if (ca[i] != i - i/2 - 64)
+  for (i = 0; i < N; ++i) {
+    ii = i % 128;
+    if (ca[i] != ii - ii/2 - 64)
       abort ();
+  }
   return 0;
 }
