@@ -1,6 +1,6 @@
 ! { dg-do run }
 ! { dg-options "-fshort-enums" }
-! { dg-options "-fshort-enums -Wl,--no-enum-size-warning" { target arm*-*-linux*eabi* } }
+! { dg-options "-fshort-enums -Wl,--no-enum-size-warning" { target arm_eabi } }
 ! Program to test enumerations when option -fshort-enums is given
 
 program main
@@ -9,7 +9,7 @@ program main
     enumerator :: red, black = 127
     enumerator blue
   end enum
-  if (red /= 0) call abort
-  if (black /= 127) call abort
-  if (blue /= 128) call abort
+  if (red /= 0) STOP 1
+  if (black /= 127) STOP 2
+  if (blue /= 128) STOP 3
 end program main
