@@ -22,11 +22,9 @@ end module
 program start
    use funcs
    implicit none
-   procedure(fun), pointer :: f
+   procedure(fun), pointer :: f ! { dg-error "Procedure pointer 'f' at .1. shall not be elemental" }
    real x(3)
    x = [1,2,3]
    f => my_dcos     ! { dg-error "Mismatch in PURE attribute" }
    write(*,*) f(x)
 end program start 
-
-! { dg-final { cleanup-modules "funcs" } }
\ No newline at end of file
