@@ -3,13 +3,13 @@
 /* { dg-options "" } */
 /* { dg-do run } */
 /* { dg-xfail-run-if "Needs OBJC2 ABI" { *-*-darwin* && { lp64 && { ! objc2 } } } { "-fnext-runtime" } { "" } } */
-#include "../objc-obj-c++-shared/Object1.h"
+#include "../objc-obj-c++-shared/TestsuiteObject.m"
 #include <stdlib.h>
 #include <objc/objc.h>
 
 extern "C" void abort(void);
 
-@interface A : Object
+@interface A : TestsuiteObject
 {
   @public
     int a;
@@ -40,4 +40,4 @@ int main()
   
   return 0;
 }
-#include "../objc-obj-c++-shared/Object1-implementation.h"
+
