@@ -1,7 +1,8 @@
 /* Test warning for non-existent selectors.  */
 /* This is the "-fgnu-runtime" variant of objc.dg/selector-1.m.  */
-/* { dg-options "-Wselector -fgnu-runtime" } */
+/* { dg-options "-Wselector" } */
 /* { dg-do compile } */
+/* { dg-skip-if "" { *-*-* } { "-fnext-runtime" } { "" } } */
 
 typedef struct objc_object { struct objc_class *class_pointer; } *id;
 typedef const struct objc_selector    *SEL;
@@ -22,5 +23,8 @@ typedef const struct objc_selector    *SEL;
   a = @selector(b1ar);
   b = @selector(bar);
 }
-@end /* { dg-warning "creating selector for nonexistent method .b1ar." } */
+@end
+
+/* FIXME: The error message should be on the correct line.  */
+/* { dg-warning "creating selector for nonexistent method .b1ar." "" { target *-*-* } 0 } */
 
