@@ -1,5 +1,5 @@
 /* Map (unsigned int) keys to (source file, line, column) triples.
-   Copyright (C) 2001-2019 Free Software Foundation, Inc.
+   Copyright (C) 2001-2020 Free Software Foundation, Inc.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,17 +26,17 @@ along with this program; see the file COPYING3.  If not see
 #include "internal.h"
 #include "hashtab.h"
 
-static void trace_include (const struct line_maps *, const line_map_ordinary *);
-static const line_map_ordinary * linemap_ordinary_map_lookup (struct line_maps *,
+static void trace_include (const line_maps *, const line_map_ordinary *);
+static const line_map_ordinary * linemap_ordinary_map_lookup (const line_maps *,
 							      location_t);
-static const line_map_macro* linemap_macro_map_lookup (struct line_maps *,
+static const line_map_macro* linemap_macro_map_lookup (const line_maps *,
 						       location_t);
 static location_t linemap_macro_map_loc_to_def_point
 (const line_map_macro *, location_t);
 static location_t linemap_macro_map_loc_to_exp_point
 (const line_map_macro *, location_t);
 static location_t linemap_macro_loc_to_spelling_point
-(struct line_maps *, location_t, const line_map_ordinary **);
+(line_maps *, location_t, const line_map_ordinary **);
 static location_t linemap_macro_loc_to_def_point (line_maps *,
 						  location_t,
 						  const line_map_ordinary **);
@@ -98,7 +98,7 @@ location_adhoc_data_update (void **slot, void *data)
 /* Rebuild the hash table from the location adhoc data.  */
 
 void
-rebuild_location_adhoc_htab (struct line_maps *set)
+rebuild_location_adhoc_htab (line_maps *set)
 {
   unsigned i;
   set->location_adhoc_data_map.htab =
@@ -113,7 +113,7 @@ rebuild_location_adhoc_htab (struct line_maps *set)
    within a location_t, without needing to use an ad-hoc location.  */
 
 static bool
-can_be_stored_compactly_p (struct line_maps *set,
+can_be_stored_compactly_p (line_maps *set,
 			   location_t locus,
 			   source_range src_range,
 			   void *data)
@@ -154,7 +154,7 @@ can_be_stored_compactly_p (struct line_maps *set,
 /* Combine LOCUS and DATA to a combined adhoc loc.  */
 
 location_t
-get_combined_adhoc_loc (struct line_maps *set,
+get_combined_adhoc_loc (line_maps *set,
 			location_t locus,
 			source_range src_range,
 			void *data)
@@ -163,8 +163,7 @@ get_combined_adhoc_loc (struct line_maps *set,
   struct location_adhoc_data **slot;
 
   if (IS_ADHOC_LOC (locus))
-    locus
-      = set->location_adhoc_data_map.data[locus & MAX_LOCATION_T].locus;
+    locus = get_location_from_adhoc_loc (set, locus);
   if (locus == 0 && data == NULL)
     return 0;
 
@@ -243,7 +242,7 @@ get_combined_adhoc_loc (struct line_maps *set,
 /* Return the data for the adhoc loc.  */
 
 void *
-get_data_from_adhoc_loc (struct line_maps *set, location_t loc)
+get_data_from_adhoc_loc (const class line_maps *set, location_t loc)
 {
   linemap_assert (IS_ADHOC_LOC (loc));
   return set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].data;
@@ -252,7 +251,7 @@ get_data_from_adhoc_loc (struct line_maps *set, location_t loc)
 /* Return the location for the adhoc loc.  */
 
 location_t
-get_location_from_adhoc_loc (struct line_maps *set, location_t loc)
+get_location_from_adhoc_loc (const class line_maps *set, location_t loc)
 {
   linemap_assert (IS_ADHOC_LOC (loc));
   return set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].locus;
@@ -261,7 +260,7 @@ get_location_from_adhoc_loc (struct line_maps *set, location_t loc)
 /* Return the source_range for adhoc location LOC.  */
 
 static source_range
-get_range_from_adhoc_loc (struct line_maps *set, location_t loc)
+get_range_from_adhoc_loc (const class line_maps *set, location_t loc)
 {
   linemap_assert (IS_ADHOC_LOC (loc));
   return set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].src_range;
@@ -271,7 +270,7 @@ get_range_from_adhoc_loc (struct line_maps *set, location_t loc)
    lookaside table, or embedded inside LOC itself.  */
 
 source_range
-get_range_from_loc (struct line_maps *set,
+get_range_from_loc (line_maps *set,
 		    location_t loc)
 {
   if (IS_ADHOC_LOC (loc))
@@ -321,8 +320,7 @@ location_t
 get_pure_location (line_maps *set, location_t loc)
 {
   if (IS_ADHOC_LOC (loc))
-    loc
-      = set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].locus;
+    loc = get_location_from_adhoc_loc (set, loc);
 
   if (loc >= LINEMAPS_MACRO_LOWEST_LOCATION (set))
     return loc;
@@ -339,12 +337,12 @@ get_pure_location (line_maps *set, location_t loc)
 /* Initialize a line map set.  */
 
 void
-linemap_init (struct line_maps *set,
+linemap_init (line_maps *set,
 	      location_t builtin_location)
 {
 #if __GNUC__ == 4 && __GNUC_MINOR__ == 2 && !defined (__clang__)
   /* PR33916, needed to fix PR82939.  */
-  memset (set, 0, sizeof (struct line_maps));
+  memset (set, 0, sizeof (line_maps));
 #else
   new (set) line_maps();
 #endif
@@ -369,7 +367,7 @@ linemap_included_from_linemap (line_maps *set, const line_map_ordinary *map)
 /* Check for and warn about line_maps entered but not exited.  */
 
 void
-linemap_check_files_exited (struct line_maps *set)
+linemap_check_files_exited (line_maps *set)
 {
   /* Depending upon whether we are handling preprocessed input or
      not, this can be a user error or an ICE.  */
@@ -386,7 +384,7 @@ linemap_check_files_exited (struct line_maps *set)
    macro maps are allocated in different memory location.  */
 
 static struct line_map *
-new_linemap (struct line_maps *set,  location_t start_location)
+new_linemap (line_maps *set,  location_t start_location)
 {
   bool macro_p = start_location >= LINE_MAP_MAX_LOCATION;
   unsigned num_maps_allocated = LINEMAPS_ALLOCATED (set, macro_p);
@@ -459,7 +457,7 @@ new_linemap (struct line_maps *set,  location_t start_location)
    maps, so any stored line_map pointers should not be used.  */
 
 const struct line_map *
-linemap_add (struct line_maps *set, enum lc_reason reason,
+linemap_add (line_maps *set, enum lc_reason reason,
 	     unsigned int sysp, const char *to_file, linenum_type to_line)
 {
   /* Generate a start_location above the current highest_location.
@@ -580,7 +578,7 @@ linemap_add (struct line_maps *set, enum lc_reason reason,
    macro expansion, FALSE otherwise.  */
 
 bool
-linemap_tracks_macro_expansion_locs_p (struct line_maps *set)
+linemap_tracks_macro_expansion_locs_p (line_maps *set)
 {
   return LINEMAPS_MACRO_MAPS (set) != NULL;
 }
@@ -609,7 +607,7 @@ linemap_tracks_macro_expansion_locs_p (struct line_maps *set)
    macro tokens anymore.  */
 
 const line_map_macro *
-linemap_enter_macro (struct line_maps *set, struct cpp_hashnode *macro_node,
+linemap_enter_macro (class line_maps *set, struct cpp_hashnode *macro_node,
 		     location_t expansion, unsigned int num_tokens)
 {
   location_t start_location
@@ -684,7 +682,7 @@ linemap_add_macro_token (const line_map_macro *map,
    the highest_location).  */
 
 location_t
-linemap_line_start (struct line_maps *set, linenum_type to_line,
+linemap_line_start (line_maps *set, linenum_type to_line,
 		    unsigned int max_column_hint)
 {
   line_map_ordinary *map = LINEMAPS_LAST_ORDINARY_MAP (set);
@@ -719,11 +717,11 @@ linemap_line_start (struct line_maps *set, linenum_type to_line,
 	  /* If the column number is ridiculous or we've allocated a huge
 	     number of location_ts, give up on column numbers
 	     (and on packed ranges).  */
-	  max_column_hint = 0;
+	  max_column_hint = 1;
 	  column_bits = 0;
 	  range_bits = 0;
 	  if (highest >= LINE_MAP_MAX_LOCATION)
-	    return 0;
+	    goto overflowed;
 	}
       else
 	{
@@ -737,6 +735,7 @@ linemap_line_start (struct line_maps *set, linenum_type to_line,
 	  max_column_hint = 1U << column_bits;
 	  column_bits += range_bits;
 	}
+
       /* Allocate the new line_map.  However, if the current map only has a
 	 single line we can sometimes just increase its column_bits instead. */
       if (line_delta < 0
@@ -766,7 +765,14 @@ linemap_line_start (struct line_maps *set, linenum_type to_line,
   /* Locations of ordinary tokens are always lower than locations of
      macro tokens.  */
   if (r >= LINE_MAP_MAX_LOCATION)
-    return 0;
+    {
+    overflowed:
+      /* Remember we overflowed.  */
+      set->highest_line = set->highest_location = LINE_MAP_MAX_LOCATION - 1;
+      /* No column numbers!  */
+      set->max_column_hint = 1;
+      return 0;
+    }
 
   set->highest_line = r;
   if (r > set->highest_location)
@@ -793,7 +799,7 @@ linemap_line_start (struct line_maps *set, linenum_type to_line,
    encoded from.  */
 
 location_t
-linemap_position_for_column (struct line_maps *set, unsigned int to_column)
+linemap_position_for_column (line_maps *set, unsigned int to_column)
 {
   location_t r = set->highest_line;
 
@@ -865,14 +871,14 @@ linemap_position_for_line_and_column (line_maps *set,
    virtual locations.  */
 
 location_t
-linemap_position_for_loc_and_offset (struct line_maps *set,
+linemap_position_for_loc_and_offset (line_maps *set,
 				     location_t loc,
 				     unsigned int column_offset)
 {
   const line_map_ordinary * map = NULL;
 
   if (IS_ADHOC_LOC (loc))
-    loc = set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].locus;
+    loc = get_location_from_adhoc_loc (set, loc);
 
   /* This function does not support virtual locations yet.  */
   if (linemap_location_from_macro_expansion_p (set, loc))
@@ -931,10 +937,10 @@ linemap_position_for_loc_and_offset (struct line_maps *set,
    ordinary or a macro map), returns that map.  */
 
 const struct line_map*
-linemap_lookup (struct line_maps *set, location_t line)
+linemap_lookup (const line_maps *set, location_t line)
 {
   if (IS_ADHOC_LOC (line))
-    line = set->location_adhoc_data_map.data[line & MAX_LOCATION_T].locus;
+    line = get_location_from_adhoc_loc (set, line);
   if (linemap_location_from_macro_expansion_p (set, line))
     return linemap_macro_map_lookup (set, line);
   return linemap_ordinary_map_lookup (set, line);
@@ -946,20 +952,20 @@ linemap_lookup (struct line_maps *set, location_t line)
    binary search.  */
 
 static const line_map_ordinary *
-linemap_ordinary_map_lookup (struct line_maps *set, location_t line)
+linemap_ordinary_map_lookup (const line_maps *set, location_t line)
 {
   unsigned int md, mn, mx;
   const line_map_ordinary *cached, *result;
 
   if (IS_ADHOC_LOC (line))
-    line = set->location_adhoc_data_map.data[line & MAX_LOCATION_T].locus;
+    line = get_location_from_adhoc_loc (set, line);
 
   if (set ==  NULL || line < RESERVED_LOCATION_COUNT)
     return NULL;
 
   mn = LINEMAPS_ORDINARY_CACHE (set);
   mx = LINEMAPS_ORDINARY_USED (set);
-  
+
   cached = LINEMAPS_ORDINARY_MAP_AT (set, mn);
   /* We should get a segfault if no line_maps have been added yet.  */
   if (line >= MAP_START_LOCATION (cached))
@@ -994,13 +1000,13 @@ linemap_ordinary_map_lookup (struct line_maps *set, location_t line)
    binary search.  */
 
 static const line_map_macro *
-linemap_macro_map_lookup (struct line_maps *set, location_t line)
+linemap_macro_map_lookup (const line_maps *set, location_t line)
 {
   unsigned int md, mn, mx;
   const struct line_map_macro *cached, *result;
 
   if (IS_ADHOC_LOC (line))
-    line = set->location_adhoc_data_map.data[line & MAX_LOCATION_T].locus;
+    line = get_location_from_adhoc_loc (set, line);
 
   linemap_assert (line >= LINEMAPS_MACRO_LOWEST_LOCATION (set));
 
@@ -1124,14 +1130,13 @@ linemap_macro_map_loc_unwind_toward_spelling (line_maps *set,
    macro expansion point.  */
 
 int
-linemap_get_expansion_line (struct line_maps *set,
+linemap_get_expansion_line (line_maps *set,
 			    location_t location)
 {
   const line_map_ordinary *map = NULL;
 
   if (IS_ADHOC_LOC (location))
-    location = set->location_adhoc_data_map.data[location
-						 & MAX_LOCATION_T].locus;
+    location = get_location_from_adhoc_loc (set, location);
 
   if (location < RESERVED_LOCATION_COUNT)
     return 0;
@@ -1152,20 +1157,18 @@ linemap_get_expansion_line (struct line_maps *set,
    SET is the line map set LOCATION comes from.  */
 
 const char*
-linemap_get_expansion_filename (struct line_maps *set,
+linemap_get_expansion_filename (line_maps *set,
 				location_t location)
 {
   const struct line_map_ordinary *map = NULL;
 
   if (IS_ADHOC_LOC (location))
-    location = set->location_adhoc_data_map.data[location
-						 & MAX_LOCATION_T].locus;
+    location = get_location_from_adhoc_loc (set, location);
 
   if (location < RESERVED_LOCATION_COUNT)
     return NULL;
 
-  location =
-    linemap_macro_loc_to_exp_point (set, location, &map);
+  linemap_macro_loc_to_exp_point (set, location, &map);
 
   return LINEMAP_FILE (map);
 }
@@ -1190,14 +1193,13 @@ linemap_map_get_macro_name (const line_map_macro *macro_map)
    header, but expanded in a non-system file.  */
 
 int
-linemap_location_in_system_header_p (struct line_maps *set,
+linemap_location_in_system_header_p (line_maps *set,
 				     location_t location)
 {
   const struct line_map *map = NULL;
 
   if (IS_ADHOC_LOC (location))
-    location = set->location_adhoc_data_map.data[location
-						 & MAX_LOCATION_T].locus;
+    location = get_location_from_adhoc_loc (set, location);
 
   if (location < RESERVED_LOCATION_COUNT)
     return false;
@@ -1236,12 +1238,11 @@ linemap_location_in_system_header_p (struct line_maps *set,
    a macro expansion, FALSE otherwise.  */
 
 bool
-linemap_location_from_macro_expansion_p (const struct line_maps *set,
+linemap_location_from_macro_expansion_p (const class line_maps *set,
 					 location_t location)
 {
   if (IS_ADHOC_LOC (location))
-    location = set->location_adhoc_data_map.data[location
-						 & MAX_LOCATION_T].locus;
+    location = get_location_from_adhoc_loc (set, location);
 
   return location >= LINEMAPS_MACRO_LOWEST_LOCATION (set);
 }
@@ -1252,13 +1253,18 @@ linemap_location_from_macro_expansion_p (const struct line_maps *set,
    virtual location of the token inside the resulting macro.  */
 
 static const struct line_map*
-first_map_in_common_1 (struct line_maps *set,
+first_map_in_common_1 (line_maps *set,
 		       location_t *loc0,
 		       location_t *loc1)
 {
   location_t l0 = *loc0, l1 = *loc1;
-  const struct line_map *map0 = linemap_lookup (set, l0),
-    *map1 = linemap_lookup (set, l1);
+  const struct line_map *map0 = linemap_lookup (set, l0);
+  if (IS_ADHOC_LOC (l0))
+    l0 = get_location_from_adhoc_loc (set, l0);
+
+  const struct line_map *map1 = linemap_lookup (set, l1);
+  if (IS_ADHOC_LOC (l1))
+    l1 = get_location_from_adhoc_loc (set, l1);
 
   while (linemap_macro_expansion_map_p (map0)
 	 && linemap_macro_expansion_map_p (map1)
@@ -1294,7 +1300,7 @@ first_map_in_common_1 (struct line_maps *set,
    return of a non-NULL result.  */
 
 static const struct line_map*
-first_map_in_common (struct line_maps *set,
+first_map_in_common (line_maps *set,
 		     location_t loc0,
 		     location_t loc1,
 		     location_t  *res_loc0,
@@ -1312,7 +1318,7 @@ first_map_in_common (struct line_maps *set,
    otherwise.  */
 
 int
-linemap_compare_locations (struct line_maps *set,
+linemap_compare_locations (line_maps *set,
 			   location_t  pre,
 			   location_t post)
 {
@@ -1368,7 +1374,7 @@ linemap_compare_locations (struct line_maps *set,
 /* Print an include trace, for e.g. the -H option of the preprocessor.  */
 
 static void
-trace_include (const struct line_maps *set, const line_map_ordinary *map)
+trace_include (const class line_maps *set, const line_map_ordinary *map)
 {
   unsigned int i = set->depth;
 
@@ -1384,7 +1390,7 @@ trace_include (const struct line_maps *set, const line_map_ordinary *map)
    This is a subroutine for linemap_resolve_location.  */
 
 static location_t
-linemap_macro_loc_to_spelling_point (struct line_maps *set,
+linemap_macro_loc_to_spelling_point (line_maps *set,
 				     location_t location,
 				     const line_map_ordinary **original_map)
 {
@@ -1418,7 +1424,7 @@ linemap_macro_loc_to_spelling_point (struct line_maps *set,
    This is a subroutine of linemap_resolve_location.  */
 
 static location_t
-linemap_macro_loc_to_def_point (struct line_maps *set,
+linemap_macro_loc_to_def_point (line_maps *set,
 				location_t location,
 				const line_map_ordinary **original_map)
 {
@@ -1460,15 +1466,14 @@ linemap_macro_loc_to_def_point (struct line_maps *set,
    This is a subroutine of linemap_resolve_location.  */
 
 static location_t
-linemap_macro_loc_to_exp_point (struct line_maps *set,
+linemap_macro_loc_to_exp_point (line_maps *set,
 				location_t location,
 				const line_map_ordinary **original_map)
 {
   struct line_map *map;
 
   if (IS_ADHOC_LOC (location))
-    location = set->location_adhoc_data_map.data[location
-						 & MAX_LOCATION_T].locus;
+    location = get_location_from_adhoc_loc (set, location);
 
   linemap_assert (set && location >= RESERVED_LOCATION_COUNT);
 
@@ -1535,14 +1540,14 @@ linemap_macro_loc_to_exp_point (struct line_maps *set,
    UNKNOWN_LOCATION or BUILTINS_LOCATION in GCC.  */
 
 location_t
-linemap_resolve_location (struct line_maps *set,
+linemap_resolve_location (line_maps *set,
 			  location_t loc,
 			  enum location_resolution_kind lrk,
 			  const line_map_ordinary **map)
 {
   location_t locus = loc;
   if (IS_ADHOC_LOC (loc))
-    locus = set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].locus;
+    locus = get_location_from_adhoc_loc (set, loc);
 
   if (locus < RESERVED_LOCATION_COUNT)
     {
@@ -1575,7 +1580,7 @@ linemap_resolve_location (struct line_maps *set,
    definition of a macro, FALSE otherwise.  */
 
 bool
-linemap_location_from_macro_definition_p (struct line_maps *set,
+linemap_location_from_macro_definition_p (line_maps *set,
 					  location_t loc)
 {
   if (IS_ADHOC_LOC (loc))
@@ -1615,7 +1620,7 @@ linemap_location_from_macro_definition_p (struct line_maps *set,
    to the map of the returned location.  */
 
 location_t
-linemap_unwind_toward_expansion (struct line_maps *set,
+linemap_unwind_toward_expansion (line_maps *set,
 				 location_t loc,
 				 const struct line_map **map)
 {
@@ -1624,7 +1629,7 @@ linemap_unwind_toward_expansion (struct line_maps *set,
   const struct line_map *resolved_map;
 
   if (IS_ADHOC_LOC (loc))
-    loc = set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].locus;
+    loc = get_location_from_adhoc_loc (set, loc);
 
   resolved_location =
     linemap_macro_map_loc_unwind_toward_spelling (set, macro_map, loc);
@@ -1655,7 +1660,7 @@ linemap_unwind_toward_expansion (struct line_maps *set,
    *MAP is set to the map of the returned location if the later is
    different from LOC.  */
 location_t
-linemap_unwind_to_first_non_reserved_loc (struct line_maps *set,
+linemap_unwind_to_first_non_reserved_loc (line_maps *set,
 					  location_t loc,
 					  const struct line_map **map)
 {
@@ -1664,7 +1669,7 @@ linemap_unwind_to_first_non_reserved_loc (struct line_maps *set,
   const line_map_ordinary *map1 = NULL;
 
   if (IS_ADHOC_LOC (loc))
-    loc = set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].locus;
+    loc = get_location_from_adhoc_loc (set, loc);
 
   map0 = linemap_lookup (set, loc);
   if (!linemap_macro_expansion_map_p (map0))
@@ -1699,7 +1704,7 @@ linemap_unwind_to_first_non_reserved_loc (struct line_maps *set,
    location is returned.  */
 
 expanded_location
-linemap_expand_location (struct line_maps *set,
+linemap_expand_location (line_maps *set,
 			 const struct line_map *map,
 			 location_t loc)
 
@@ -1709,9 +1714,8 @@ linemap_expand_location (struct line_maps *set,
   memset (&xloc, 0, sizeof (xloc));
   if (IS_ADHOC_LOC (loc))
     {
-      xloc.data
-	= set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].data;
-      loc = set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].locus;
+      xloc.data = get_data_from_adhoc_loc (set, loc);
+      loc = get_location_from_adhoc_loc (set, loc);
     }
 
   if (loc < RESERVED_LOCATION_COUNT)
@@ -1749,7 +1753,7 @@ linemap_expand_location (struct line_maps *set,
    dump a macro map, false otherwise.  */
 
 void
-linemap_dump (FILE *stream, struct line_maps *set, unsigned ix, bool is_macro)
+linemap_dump (FILE *stream, class line_maps *set, unsigned ix, bool is_macro)
 {
   const char *const lc_reasons_v[LC_HWM]
       = { "LC_ENTER", "LC_LEAVE", "LC_RENAME", "LC_RENAME_VERBATIM",
@@ -1805,7 +1809,7 @@ linemap_dump (FILE *stream, struct line_maps *set, unsigned ix, bool is_macro)
    stream STREAM. SET is the line map set LOC comes from.  */
 
 void
-linemap_dump_location (struct line_maps *set,
+linemap_dump_location (line_maps *set,
 		       location_t loc,
 		       FILE *stream)
 {
@@ -1815,7 +1819,7 @@ linemap_dump_location (struct line_maps *set,
   int l = -1, c = -1, s = -1, e = -1;
 
   if (IS_ADHOC_LOC (loc))
-    loc = set->location_adhoc_data_map.data[loc & MAX_LOCATION_T].locus;
+    loc = get_location_from_adhoc_loc (set, loc);
 
   if (loc == 0)
     return;
@@ -1855,7 +1859,7 @@ linemap_dump_location (struct line_maps *set,
    location emitted for that file.  */
 
 bool
-linemap_get_file_highest_location (struct line_maps *set,
+linemap_get_file_highest_location (line_maps *set,
 				   const char *file_name,
 				   location_t *loc)
 {
@@ -1893,7 +1897,7 @@ linemap_get_file_highest_location (struct line_maps *set,
    parts of the line table SET.  */
 
 void
-linemap_get_statistics (struct line_maps *set,
+linemap_get_statistics (line_maps *set,
 			struct linemap_stats *s)
 {
   long ordinary_maps_allocated_size, ordinary_maps_used_size,
@@ -1957,7 +1961,7 @@ linemap_get_statistics (struct line_maps *set,
    specifies how many macro maps to dump.  */
 
 void
-line_table_dump (FILE *stream, struct line_maps *set, unsigned int num_ordinary,
+line_table_dump (FILE *stream, class line_maps *set, unsigned int num_ordinary,
 		 unsigned int num_macro)
 {
   unsigned int i;
@@ -2002,7 +2006,8 @@ rich_location::rich_location (line_maps *set, location_t loc,
   m_have_expanded_location (false),
   m_fixit_hints (),
   m_seen_impossible_fixit (false),
-  m_fixits_cannot_be_auto_applied (false)
+  m_fixits_cannot_be_auto_applied (false),
+  m_path (NULL)
 {
   add_range (loc, SHOW_RANGE_WITH_CARET, label);
 }
