@@ -4,9 +4,16 @@
 
 package cgotest
 
-import "testing"
+import (
+	"runtime"
+	"testing"
+)
 
-func TestSetgid(t *testing.T)  { testSetgid(t) }
+func TestSetgid(t *testing.T) {
+	if runtime.GOOS == "android" {
+		t.Skip("unsupported on Android")
+	}
+	testSetgid(t)
+}
 func Test6997(t *testing.T)    { test6997(t) }
 func TestBuildID(t *testing.T) { testBuildID(t) }
-func Test9400(t *testing.T)    { test9400(t) }
