@@ -1,6 +1,6 @@
 // Filesystem operation utilities -*- C++ -*-
 
-// Copyright (C) 2014-2019 Free Software Foundation, Inc.
+// Copyright (C) 2014-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -104,7 +104,16 @@ namespace __gnu_posix
 #endif
 
   inline int rename(const wchar_t* oldname, const wchar_t* newname)
-  { return _wrename(oldname, newname); }
+  {
+    if (MoveFileExW(oldname, newname,
+		    MOVEFILE_REPLACE_EXISTING | MOVEFILE_COPY_ALLOWED))
+      return 0;
+    if (GetLastError() == ERROR_ACCESS_DENIED)
+      errno = EACCES;
+    else
+      errno = EIO;
+    return -1;
+  }
 
   inline int truncate(const wchar_t* path, _off64_t length)
   {
