@@ -1,6 +1,6 @@
 // { dg-do compile { target c++11 } }
 
-// Copyright (C) 2012-2019 Free Software Foundation, Inc.
+// Copyright (C) 2012-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -214,6 +214,7 @@ static_assert(is_type<std::common_type<void(&&)(), void(&)()>,
 static_assert(is_type<std::common_type<void(&&)(), void(&&)()>,
 	      void(*)()>(), "");
 static_assert(is_type<std::common_type<ImplicitTo<int>, int>, int>(), "");
+static_assert(is_type<std::common_type<const ImplicitTo<int>, int>, int>(), "");
 static_assert(is_type<std::common_type<ImplicitTo<int>, ImplicitTo<int>>,
 	      ImplicitTo<int>>(), "");
 static_assert(is_type<std::common_type<ImplicitTo<int>, int,
@@ -260,19 +261,22 @@ static_assert(is_type<std::common_type<volatile Ukn&&, volatile Ukn&&>,
 	      Ukn>(), "");
 
 static_assert(is_type<std::common_type<X1, X2>, RX12>(), "");
+static_assert(is_type<std::common_type<const X1, X2>, RX12>(), "");
+static_assert(is_type<std::common_type<X1&, const X2>, RX12>(), "");
+static_assert(is_type<std::common_type<const X1&, const X2&>, RX12>(), "");
 static_assert(is_type<std::common_type<X2, X1>, RX21>(), "");
 
 static_assert(is_type<std::common_type<X1, X2, X1>, Y1>(), "");
 static_assert(is_type<std::common_type<X2, X1, X1>, Y3>(), "");
 
 static_assert(is_type<std::common_type<X1, X1, X2>, RX12>(), "");
+static_assert(is_type<std::common_type<X1&, const X1, const X2&&>, RX12>(), "");
 static_assert(is_type<std::common_type<X1, X1, X2, X1>, Y1>(), "");
 
 static_assert(!has_type<std::common_type<>>(), "");
 static_assert(!has_type<std::common_type<int, S>>(), "");
 static_assert(!has_type<std::common_type<U, S>>(), "");
 static_assert(!has_type<std::common_type<U, U2>>(), "");
-static_assert(!has_type<std::common_type<const ImplicitTo<int>, int>>(), "");
 static_assert(!has_type<std::common_type<PrivateImplicitTo<int>, int>>(), "");
 static_assert(!has_type<std::common_type<const PrivateImplicitTo<int>,
 	      int>>(), "");
